from __future__ import absolute_import, unicode_literals
import base64, os

from . import Environment

__all__ = ['DjangoEnvironment']

class DjangoEnvironment(Environment):
    def __init__(self, db_scheme_map = None, email_backend_map = None, *args, **kwargs):
        super(DjangoEnvironment, self).__init__(*args, **kwargs)

        self.db_scheme_map = {
            'postgresql': 'django.db.backends.postgresql_psycopg2',
            'postgis':    'django.contrib.gis.db.backends.postgis',
            'mysql':      'django.db.backends.mysql',
        }

        self.email_backend_map = {
            'smtp':    'django.core.mail.backends.smtp.EmailBackend',
            'console': 'django.core.mail.backends.console.EmailBackend',
            'file':    'django.core.mail.backends.filebased.EmailBackend',
            'locmem':  'django.core.mail.backends.locmem.EmailBackend',
        }

        self.db_scheme_map.update(dict(db_scheme_map or {}))
        self.email_backend_map.update(dict(email_backend_map or {}))

    def get_database_url(self, key, default, OPTIONS = None, TEST = None, **kwargs):
        parsed = self.get_url(key, default)

        db = {
            'ENGINE':  self.db_scheme_map[parsed.scheme],
            'NAME':    parsed.path[1:],  # skip initial /
            'OPTIONS': {},
            'TEST':    {},
        }

        if parsed.hostname:
            db['HOST'] = parsed.hostname

        if parsed.port:
            db['PORT'] = int(parsed.port)

        if parsed.username:
            db['USER'] = parsed.username

        if parsed.password:
            db['PASSWORD'] = parsed.password

        db.update(kwargs)
        db['OPTIONS'].update(OPTIONS or {})
        db['TEST'].update(TEST or {})

        return db

    def get_email_backend(self, key, default):
        return self.email_backend_map[self.get(key, default)]

    def get_temporary_key(self):
        return base64.b64encode(os.urandom(32))

    def get_secret_key(self, key):
        return self.get(key, self.get_temporary_key)
